local lib = TeamEquipCompare;

local function TEC_OnTooltipSetItem(tooltip, ...)
	if (not lib.db.global.requireCtrlKey) or ((lib.db.global.requireCtrlKey) and IsControlKeyDown()) then
		-- get info for tooltip item
		tt_name, tt_link = tooltip:GetItem();
		_, _, _, _, tt_reqLevel, tt_class, tt_subclass, _, tt_equipSlot, _, _ = GetItemInfo(tt_link);
		local slot = TEC_Slot_Lookup[tt_equipSlot];

		-- make sure the slot is one we're interested in
		if (slot and slot.ids) then
			-- add heading line with spacing
			tooltip:AddLine(" ");
			if (not IsInGroup()) then
				-- simple message if player is not in a group
				tooltip:AddDoubleLine("Team Equip Compare:", "|cffff0000[Not in Group]|r");
				tooltip:AddLine(" ");
				return;
			else
				tooltip:AddLine("Team Equip Compare:");
				tooltip:AddLine(" ");
			end
			
			-- loop thru party members in alphabetical order
			local sortedNames = tec_keys(lib.party, true);
			for _, unit in pairs(sortedNames) do
				local class = TEC_Class_Lookup[lib.party[unit].class];
				local classColor = TEC_ClassColor_Lookup[lib.party[unit].class];
				local isUsable = TEC_ItemIsUsable(lib.party[unit], class, tt_reqLevel, tt_class, tt_subclass);
				local playerTag = format("|c%s%s|r:", classColor, lib.party[unit].name);

				if (not isUsable) then
					-- add player name with note that tooltip item is unusable
					tooltip:AddDoubleLine(playerTag, "|cffff0000[Unable to Equip]|r");
					tooltip:AddLine(" ");
				else
					-- add party member heading
					tooltip:AddLine(playerTag);

					-- compare item with each potential slot (weapons, trinkets and fingers have two slots)
					for ii = 1, #slot.ids do
						-- grab item from syncronized party member data
						local link = lib.party[unit].gear[slot.ids[ii]];
						if (link) then
							-- get party member item data
							local name, _, quality, ilevel, _, _, _, _, _, _, _ = GetItemInfo(link);
							if (name) then
								-- get item text, colored by item quality
								local itemTag = format("     |c%s[%d] %s|r", TEC_QualityColor_Lookup[quality], ilevel, name)

								-- get stat comparison values, or message if not usable
								local statTip = GetStatLine(isUsable, tt_link, link);
								
								-- add item line to the tooltip
								tooltip:AddDoubleLine(itemTag, statTip);
							end
						end
					end

					-- add spacer between party members
					tooltip:AddLine(" ");
				end
			end
		end
	end
end

local function TEC_OnTooltipCleared(tooltip, ...)
end

-- function can't be local, presumably since it's called from GlobalTooltip
function TEC_ItemIsUsable(unit, class, reqLevel, itemClass, itemSubclass)
	if (reqLevel > unit.level) then
		return false;
	elseif (not class) then
		return false;
	elseif (not class[itemClass]) then
		return false;
	elseif (not class[itemClass][itemSubclass]) then
		return false;
	elseif (class[itemClass][itemSubclass] > unit.level) then
		return false;
	else
		return true;
	end
end

-- function can't be local, presumably since it's called from GlobalTooltip
function GetStatLine(isUsable, itemLink, equippedLink)
	local result = "";

	if (isUsable) then
		local statDelta = GetItemStatDelta(itemLink, equippedLink);
		for stat, value in pairs(statDelta) do
			-- separate stat items with a comma
			if (#result > 0) then result = result .. ", " end
			
			-- get friendly name string from lookup table
			statName = TEC_Stat_Lookup[stat];
			if (not statName) then statName = tostring(stat) end

			-- color comparison text depending on +/-
			if (value > 0) then
				result = result..format("|cff00ff00+%d|r %s", value, statName);
			elseif (value < 0) then
				result = result..format("|cffff0000%d|r %s", value, statName);
			end
		end
		
		if (#result == 0) then
			result = "[Identical]";
		end
	end

	return result;
end

-- hook into the game tooltip
GameTooltip:HookScript("OnTooltipSetItem", TEC_OnTooltipSetItem);
GameTooltip:HookScript("OnTooltipCleared", TEC_OnTooltipCleared);